<?php
/*******************************************************************************
 *
 * LEIDEN OPEN VARIATION DATABASE (LOVD)
 *
 * Created     : 2009-09-25
 * Modified    : 2009-12-07
 * For LOVD    : 2.0-23
 *
 * Access      : Public
 * Purpose     : RESTful API/Web Service for LOVD.
 *               Supported:
 *               /api/rest.php/variants/{{ GENE }}
 *               /api/rest.php/variants/{{ GENE }}/{{ ID }}
 *               /api/rest.php/variants/{{ GENE }}?search_position=c.1234
 *               /api/rest.php/variants/{{ GENE }}?search_position=c.1234+56_2345-67 (c.1234%2B56_2345-67)
 *               /api/rest.php/variants/{{ GENE }}?search_position=g.12345678
 *               /api/rest.php/variants/{{ GENE }}?search_Variant%2FDNA=c.1234C>G (c.1234C%3EG)
 *               /api/rest.php/variants/{{ GENE }}?search_Variant%2FDBID=DMD_01234
 *               /api/rest.php/variants/{{ GENE }}?format=text/bed
 *               /api/rest.php/variants/{{ GENE }}?format=text/bed&visibility=2
 *               /api/rest.php/genes
 *               /api/rest.php/genes/{{ GENE }}
 *               /api/rest.php/genes?search_symbol=DMD
 *
 * Copyright   : 2004-2009 Leiden University Medical Center; http://www.LUMC.nl/
 * Programmer  : Ing. Ivo F.A.C. Fokkema <I.F.A.C.Fokkema@LUMC.nl>
 * Last edited : Ing. Ivo F.A.C. Fokkema <I.F.A.C.Fokkema@LUMC.nl>
 *
 *
 * This file is part of LOVD.
 *
 * LOVD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * LOVD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with LOVD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *************/

define('ROOT_PATH', '../');
require ROOT_PATH . 'inc-init.php';

// Will currently only allow GET.
if (!in_array($_SERVER['REQUEST_METHOD'], array('GET'))) {
    header('HTTP/1.1 501 Not Implemented');
}

// Parse URL to see what we need to do.
$aARGS = array(
                'sDataType',
                'sSymbol',
                'nID',
              );
$sDataType = $sSymbol = $nID = '';

$aPATH_INFO = explode('/', $_SERVER['PATH_INFO']);
$i = 0;
foreach ($aPATH_INFO as $sPath) {
    if (!$sPath) {
        continue;
    }
    $$aARGS[$i] = $sPath;
    $i ++;
}
if (!$i) { // No data type given.
    header('HTTP/1.1 400 Bad Request');
    die('Too few parameters.');
} elseif (!in_array($sDataType, array('variants', 'genes'))) { // Wrong data type given.
    header('HTTP/1.1 400 Bad Request');
    die('Requested data type not known.');
} elseif ($sDataType == 'variants' && $i < 2) { // Variants, but no gene selected.
    header('HTTP/1.1 400 Bad Request');
    die('Too few parameters.');
}
// Now we've got $sDataType, $sSymbol, $nID filled in, if data is available.





// Check if gene exists.
if ($sSymbol && !in_array($sSymbol, lovd_getGeneList())) {
    header('HTTP/1.1 404 Not Found');
    die('This gene does not exist.');
}





// Need some libraries.
require ROOT_PATH . 'inc-lib-api.php';

// Depending on the requested data type, we need to segment the code here.
if ($sDataType == 'variants') {
    // Check if the DNA and DBID fields are actually there (should always be the case except in modified LOVD instances).
    require ROOT_PATH . 'class/currdb.php';
    $_CURRDB = new CurrDB(true, $sSymbol);
    if ((!$_CURRDB->colExists('Variant/DNA') || !$_CURRDB->colExists('Variant/DBID')) && $sDataType == 'variants') {
        header('HTTP/1.1 503 Service Unavailable');
        die('This gene does not have the Variant/DNA or the Variant/DBID fields enabled, crucial for the API.');
    }

    if ($nID && !preg_match('/^[0-9]+$/', $nID)) {
        header('HTTP/1.1 404 Not Found');
        die(ucfirst($sDataType) . ' ID does not exist.');
    }

    // Get chromosome, reference sequence, and other data.
    list($sChromosome, $sRefSeq, $nPositionMRNAStart, $nPositionMRNAEnd, $nPositionCDSEnd) = mysql_fetch_row(mysql_query('SELECT chrom_location, refseq_mrna, c_position_mrna_start, c_position_mrna_end, c_position_cds_end FROM ' . TABLE_DBS . ' WHERE symbol = "' . $sSymbol . '"'));
    $sChromosome = preg_replace('/^([0-9]+|[XY])[pq0-9.-]+$/', "$1", $sChromosome);  // That regexp is not completely correct (not limiting enough), but we don't need that anyway.

    if (!empty($_GET['format']) && $_GET['format'] == 'text/bed') {
        // We're exporting a BED file for a Genome Browser.
        if ($sRefSeq) {
            $sRefSeqAcc = substr($sRefSeq, 0, strpos($sRefSeq, '.'));
            $sQ = 'SELECT LEAST(v.g_position_start, v.g_position_end), GREATEST(v.g_position_start, v.g_position_end), v.type, v.`Variant/DNA` FROM ' . TABLEPREFIX . '_' . $sSymbol . '_variants AS v LEFT JOIN ' . TABLE_PAT2VAR . ' AS p2v ON (p2v.symbol = "' . $sSymbol . '" AND v.variantid = p2v.variantid) WHERE p2v.status >= ' . STATUS_MARKED . ' AND v.g_position_start != 0 GROUP BY v.`Variant/DNA`';
        } else {
            // Not mappable!
            header('HTTP/1.1 503 Service Unavailable');
            die('This gene does not have a NM reference sequence associated to it, crucial for mapping variants to the genome.');
        }

    } else {
        // First build query.
        $sQ = 'SELECT v.variantid, v.c_position_start, v.c_position_start_intron, v.c_position_end, v.c_position_end_intron, v.g_position_start, v.g_position_end, v.`Variant/DNA`, v.`Variant/DBID` FROM ' . TABLEPREFIX . '_' . $sSymbol . '_variants AS v LEFT JOIN ' . TABLE_PAT2VAR . ' AS p2v ON (p2v.symbol = "' . $sSymbol . '" AND v.variantid = p2v.variantid) WHERE p2v.status >= ' . STATUS_MARKED;
        $bSearching = false;
        if ($nID) {
            $sFeedType = 'entry';
            $sQ .= ' AND v.variantid = "' . $nID . '"';
        } else {
            $sFeedType = 'feed';
            // Ok, are we searching then?
            $aSearchableFields = array('position', 'Variant/DNA', 'Variant/DBID');
            foreach ($aSearchableFields as $sField) {
                if (!empty($_GET['search_' . $sField])) {
                    $bSearching = true;
                    if ($sField == 'position') {
                        if ($sRefSeq && (preg_match('/^(g)\.([0-9]+)(_([0-9]+))?$/', $_GET['search_' . $sField], $aRegs) || preg_match('/^(c)\.([*-]?[0-9]+([+-][du]?[0-9]+)?)(_([*-]?[0-9]+([+-][du]?[0-9]+)?))?$/', $_GET['search_' . $sField], $aRegs))) {
                            // $aRegs numbering:        1    2       3 4                                                                   1    2           3                  4 5           6
                            // Mapping is only possible if there is a Reference Sequence.
                            if ($aRegs[1] == 'g') {
                                if (empty($aRegs[3])) {
                                    // No range. Absolute location.
                                    $aRegs[4] = $aRegs[2];
                                }
                                $sQ .= ' AND ' . $aRegs[1] . '_position_start = "' . $aRegs[2] . '" AND ' . $aRegs[1] . '_position_end = "' . $aRegs[4] . '"';
                            } else {
                                $aStart = lovd_convertDNAPositionToDB($nPositionMRNAStart, $nPositionMRNAEnd, $nPositionCDSEnd, $aRegs[2]);
                                if (empty($aRegs[4])) {
                                    $aEnd = $aStart;
                                } else {
                                    $aEnd = lovd_convertDNAPositionToDB($nPositionMRNAStart, $nPositionMRNAEnd, $nPositionCDSEnd, $aRegs[5]);
                                }
                                $sQ .= ' AND ' . $aRegs[1] . '_position_start = "' . $aStart[0] . '" AND ' . $aRegs[1] . '_position_start_intron = "' . $aStart[1] . '" AND ' . $aRegs[1] . '_position_end = "' . $aEnd[0] . '" AND ' . $aRegs[1] . '_position_end_intron = "' . $aEnd[1] . '"';
                            }
                        } else {
                            // This does a first match; trying to find the position at the start of the DNA field. Later this match will be made more accurate!
                            $sQ .= ' AND REPLACE(REPLACE(REPLACE(REPLACE(v.`Variant/DNA`, "[", ""), "(", ""), ")", ""), "?", "") LIKE "' . $_GET['search_' . $sField] . '%"';
                        }
                    } elseif ($sField == 'Variant/DNA') {
                        // This matches regardless of the characters (, ) and ?.
                        $sQ .= ' AND REPLACE(REPLACE(REPLACE(v.`' . $sField . '`, "(", ""), ")", ""), "?", "") = "' . str_replace(array('(', ')', '?'), ' ', $_GET['search_' . $sField]) . '"';
                    } elseif ($sField == 'Variant/DBID') {
                        $sQ .= ' AND v.`' . $sField . '` LIKE "' . $_GET['search_' . $sField] . '%"';
                    } else {
                        $sQ .= ' AND v.`' . $sField . '` = "' . $_GET['search_' . $sField] . '"';
                    }
                }
            }
        }
        $sQ .= ' GROUP BY variantid ORDER BY v.sort, v.`Variant/DNA`';
    }



} elseif ($sDataType == 'genes') {
    // Listing or simple request on gene symbol.
    // First build query.
    $sQ = 'SELECT g.symbol, g.gene, g.chrom_location, g.refseq_genomic, g.refseq_mrna, g.refseq_build, g.created_date, g.updated_date, u.name AS created_by FROM ' . TABLE_DBS . ' AS g LEFT JOIN ' . TABLE_USERS . ' AS u ON (g.created_by = u.userid) WHERE 1=1';
    $bSearching = false;
    if ($sSymbol) {
        $sFeedType = 'entry';
        $sQ .= ' AND symbol = "' . $sSymbol . '"';
    } else {
        $sFeedType = 'feed';
        // Ok, are we searching then?
        $aSearchableFields = array('symbol'); // Allow search on chromosomal location, too?
        foreach ($aSearchableFields as $sField) {
            if (!empty($_GET['search_' . $sField])) {
                $bSearching = true;
                if ($sField == 'symbol') {
                    $sQ .= ' AND (symbol = "' . $_GET['search_' . $sField] . '" OR symbol LIKE "' . $_GET['search_' . $sField] . '\_%")';
                }
            }
        }
    }
    $sQ .= ' ORDER BY symbol';
}
$q = mysql_query($sQ);
$n = mysql_num_rows($q);

if ($n) {
    header('HTTP/1.1 200 OK');
} else {
    header('HTTP/1.1 404 Not Found');
    if ($nID) {
        // Really requested a (variant) ID. Goodbye.
        die(ucfirst($sDataType) . ' ID does not exist.');
    }
}

if ($sDataType == 'variants' && !empty($_GET['format']) && $_GET['format'] == 'text/bed') {
    // We're exporting a BED file for a Genome Browser.
    // This code structure is getting pretty bad, by the way.
    $aVariantTypeColors =
             array(
                    'substr' => '204,0,255',
                    '>'      => '204,0,255', // This one can be removed later.
                    'del'    => '0,0,255',
                    'ins'    => '0,153,0',
                    'dup'    => '255,153,0',
                    ''       => '0,0,0', // Backup, for non-matching variants.
                  );

    // Print header.
    header('Content-type: text/plain; charset=ISO-8859-1');
    print('track name="Variants in LOVD ' . $sSymbol . ' database" description="Variants in LOVD ' . $sSymbol . ' database" visibility=' . (!empty($_GET['visibility']) && is_numeric($_GET['visibility'])? $_GET['visibility'] : 3) . ' itemRgb="On" db="hg19" offset=-1 url="' . ($_CONF['location_url']? $_CONF['location_url'] : PROTOCOL . $_SERVER['HTTP_HOST'] . lovd_cleanDirName(dirname($_SERVER['SCRIPT_NAME']) . '/' . ROOT_PATH)) . 'variants.php?select_db=' . $sSymbol . '&action=search_all&trackid=$$' . '"' . "\n\n");
    while ($r = mysql_fetch_row($q)) {
        list($nPositionStart, $nPositionEnd, $sVariantType, $sDNA) = $r;
        if (!isset($aVariantTypeColors[$sVariantType])) {
            $sVariantType = '';
        }
        $sVariantTypeColor = $aVariantTypeColors[$sVariantType];

        // Print the data.
        print('chr' . $sChromosome . "\t" . $nPositionStart . "\t" . ($nPositionEnd+1) . "\t" . $sSymbol . ':' . preg_replace('/\s+/', '', $sDNA) . "\t" . '0' . "\t" . '+' . "\t" . $nPositionStart . "\t" . ($nPositionEnd+1) . "\t" . $sVariantTypeColor . "\n");
    }
    exit;
}

// Start feed class.
require ROOT_PATH . 'class/feeds.php';
if ($sFeedType == 'feed') {
    if ($sDataType == 'variants') {
        $sTitle = ($bSearching? ($n? 'R' : 'No r') . 'esults for your query of' : 'Listing of all public variants in') . ' the ' . $sSymbol . ' gene database';
    } elseif ($sDataType == 'genes') {
        $sTitle = ($bSearching? ($n? 'R' : 'No r') . 'esults for your query of' : 'Listing of all genes in') . ' the database';
    }
    $sLink = ($_CONF['location_url']? $_CONF['location_url'] : PROTOCOL . $_SERVER['HTTP_HOST'] . lovd_cleanDirName(dirname($_SERVER['SCRIPT_NAME']) . '/' . ROOT_PATH)) . 'api/rest.php/' . $sDataType . ($sSymbol? '/' . $sSymbol : '');
    $sID   = 'tag:' . $_SERVER['HTTP_HOST'] . ',' . $_STAT['date_install'] . ':' . $_STAT['signature'] . '/REST_api';
} else {
    $sTitle = $sLink = $sID = '';
}
$_FEED = new Feed($sFeedType, $sTitle, $sLink, $sID, 'atom');

// Now we will create entries in the feed with the fetched data.
if ($sDataType == 'variants') {
    while ($zData = mysql_fetch_assoc($q)) {
        // Prepare other fields to be included.
        $sTitle = substr($sSymbol, 0, strpos($sSymbol . '_', '_')) . ':' . $zData['Variant/DNA'];
        if ($sFeedType == 'feed') {
            $sSelfURL = ($_CONF['location_url']? $_CONF['location_url'] : PROTOCOL . $_SERVER['HTTP_HOST'] . lovd_cleanDirName(dirname($_SERVER['SCRIPT_NAME']) . '/' . ROOT_PATH)) . 'api/rest.php/variants/' . $sSymbol . '/' . $zData['variantid'];
        } else {
            $sSelfURL = '';
        }
        // FIXME; Variant/DBID moet geisoleerd worden. Daar hadden we code voor, toch?
        $sAltURL               = ($_CONF['location_url']? $_CONF['location_url'] : PROTOCOL . $_SERVER['HTTP_HOST'] . lovd_cleanDirName(dirname($_SERVER['SCRIPT_NAME']) . '/' . ROOT_PATH)) . 'variants.php?select_db=' . $sSymbol . '&amp;action=search_unique&amp;search_Variant%2FDBID=' . rawurlencode($zData['Variant/DBID']);
        $zData['created_date'] = '1970-01-01 00:00:00';
        $zData['updated_date'] = '1970-01-01 00:00:00';
        $sID                   = 'tag:' . $_SERVER['HTTP_HOST'] . ',' . substr($zData['created_date'], 0, 10) . ':' . $sSymbol . '/' . $zData['variantid'];
        $zData['created_by']   = 'Unknown';
        $sContributors         = 'Unknown';
        if ($sRefSeq && $zData['c_position_start']) {
            $sDNAStart = lovd_convertDNAPositionToHR($nPositionMRNAStart, $nPositionMRNAEnd, $nPositionCDSEnd, $zData['c_position_start'], $zData['c_position_start_intron']);
            if ($zData['c_position_start'] == $zData['c_position_end']) {
                $sDNAEnd = $sDNAStart;
            } else {
                $sDNAEnd = lovd_convertDNAPositionToHR($nPositionMRNAStart, $nPositionMRNAEnd, $nPositionCDSEnd, $zData['c_position_end'], $zData['c_position_end_intron']);
            }
            $sPosition_mRNA    = $sRefSeq . ':c.' . $sDNAStart . ($zData['c_position_end'] == $zData['c_position_start']? '' : '_' . $sDNAEnd);
            $sPosition_genomic = 'chr' . $sChromosome . ':' . $zData['g_position_start'] . ($zData['g_position_end'] == $zData['g_position_start']? '' : '_' . $zData['g_position_end']);
        } else {
            $sPosition_mRNA    = lovd_variantToPosition($zData['Variant/DNA']);
            $sPosition_genomic = 'chr' . $sChromosome . ':?';
        }
        // Really not quite the best solution, but it kind of works. $n is decreased and if there are no more matches, it will give a 404 anyway. Still has a misleading Feed title, though!
        // FIXME; do we want to fix that by implementing a $_FEED->setTitle()?
        if (!$sRefSeq && $bSearching && !empty($_GET['search_position']) && $_GET['search_position'] != $sPosition_mRNA) {
            // This was a false positive! (only when there is no Reference Sequence LOVD will try the DNA field to find the position) Partial match that should not have been reported. Byeeeeeeee...
            $n --; // Does not really matter at this point.
            continue;
        }
        $sContent = 'symbol:' . $sSymbol . "\n" .
                    'id:' . $zData['variantid'] . "\n" .
                    'position_mRNA:' . $sPosition_mRNA . "\n" .
                    'position_genomic:' . $sPosition_genomic . "\n" .
                    'Variant/DNA:' . $zData['Variant/DNA'] . "\n" .
                    'Variant/DBID:' . $zData['Variant/DBID'];
        $_FEED->addEntry($sTitle, $sSelfURL, $sAltURL, $sID, $zData['created_by'], $zData['created_date'], $sContributors, $zData['updated_date'], '', 'text', $sContent);
    }



} elseif ($sDataType == 'genes') {
    while ($zData = mysql_fetch_assoc($q)) {
        // Prepare other fields to be included.
        $sTitle = $zData['symbol'];
        if ($sFeedType == 'feed') {
            $sSelfURL = ($_CONF['location_url']? $_CONF['location_url'] : PROTOCOL . $_SERVER['HTTP_HOST'] . lovd_cleanDirName(dirname($_SERVER['SCRIPT_NAME']) . '/' . ROOT_PATH)) . 'api/rest.php/genes/' . $zData['symbol'];
        } else {
            $sSelfURL = '';
        }
        // FIXME; Variant/DBID moet geisoleerd worden. Daar hadden we code voor, toch?
        $sAltURL             = ($_CONF['location_url']? $_CONF['location_url'] : PROTOCOL . $_SERVER['HTTP_HOST'] . lovd_cleanDirName(dirname($_SERVER['SCRIPT_NAME']) . '/' . ROOT_PATH)) . 'home.php?select_db=' . $zData['symbol'];
        $sID                 = 'tag:' . $_SERVER['HTTP_HOST'] . ',' . substr($zData['created_date'], 0, 10) . ':' . $zData['symbol'];
        $sContributors       = '';
        $qCurators = mysql_query('SELECT u.name FROM ' . TABLE_USERS . ' AS u LEFT JOIN ' . TABLE_CURATES . ' AS c USING (userid) WHERE c.symbol = "' . $zData['symbol'] . '"');
        while (list($sName) = mysql_fetch_row($qCurators)) {
            $sContributors .= ($sContributors? ', ' : '') . $sName;
        }
        $sContent = 'id:' . $zData['symbol'] . "\n" .
                    'symbol:' . substr($zData['symbol'], 0, strpos($zData['symbol'] . '_', '_')) . "\n" .
                    'name:' . $zData['gene'] . "\n" .
                    'chromosome_location:' . $zData['chrom_location'] . "\n" .
                    'refseq_genomic:' . $zData['refseq_genomic'] . "\n" .
                    'refseq_mrna:' . $zData['refseq_mrna'] . "\n" .
                    'refseq_build:' . $zData['refseq_build'];
        $_FEED->addEntry($sTitle, $sSelfURL, $sAltURL, $sID, $zData['created_by'], $zData['created_date'], $sContributors, $zData['updated_date'], '', 'text', $sContent);
    }
}

if (!$n) {
    // This happens if searching on position and there is a partial match. MySQL returns a false positive which has been filtered out now.
    header('HTTP/1.1 404 Not Found'); // This will replace the previous 200 OK status!
}

$_FEED->publish();
?>
